indexing

	description:

		"Cursors for data structure traversals"

	library: "Gobo Eiffel Structure Library"
	copyright: "Copyright (c) 1999-2001, Eric Bezault and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2003/02/07 12:58:08 $"
	revision: "$Revision: 1.5 $"

deferred class DS_CURSOR [G]

inherit

	ANY
		redefine
			copy, is_equal
		end

feature -- Access

	item: G is
			-- Item at cursor position
		require
			not_off: not off
		do
			Result := container.cursor_item (Current)
		end

	container: DS_TRAVERSABLE [G] is
			-- Data structure traversed
		deferred
		end

feature -- Status report

	off: BOOLEAN is
			-- Is there no item at cursor position?
		do
			Result := container.cursor_off (Current)
		end

	same_position (other: like Current): BOOLEAN is
			-- Is current cursor at same position as `other'?
		require
			other_not_void: other /= Void
		do
			Result := container.cursor_same_position (Current, other)
		end

	valid_cursor (other: like Current): BOOLEAN is
			-- Is `other' a valid cursor according
			-- to current traversal strategy?
		require
			other_not_void: other /= Void
		do
			Result := container.valid_cursor (other)
		ensure
			Result implies container.valid_cursor (other)
		end

	is_valid: BOOLEAN is
			-- Is cursor valid?
		obsolete
			"Backward compatibility with Gobo 1.4"
		do
			Result := True
		ensure
			definition: Result = True
		end

feature -- Cursor movement

	go_to (other: like Current) is
			-- Move cursor to `other''s position.
		require
			other_not_void: other /= Void
			other_valid: valid_cursor (other)
		do
			container.cursor_go_to (Current, other)
		ensure
			same_position: same_position (other)
		end

feature -- Duplication

	copy (other: like Current) is
			-- Copy `other' to current cursor.
		do
			if container /= Void and then not off then
				container.remove_traversing_cursor (Current)
			end
			standard_copy (other)
			next_cursor := Void
			if not off then
				container.add_traversing_cursor (Current)
			end
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- Are `other' and current cursor at the same position?
		do
			if same_type (other) then
				Result := same_position (other)
			end
		end

feature {DS_TRAVERSABLE} -- Implementation

	next_cursor: DS_CURSOR [G]
			-- Next cursor
			-- (Used by `container' to keep track of traversing
			-- cursors (i.e. cursors associated with `container'
			-- and which are not currently `off').)

	set_next_cursor (a_cursor: like next_cursor) is
			-- Set `next_cursor' to `a_cursor'.
		do
			next_cursor := a_cursor
		ensure
			next_cursor_set: next_cursor = a_cursor
		end

invariant

	container_not_void: container /= Void
	empty_constraint: container.is_empty implies off

end
